var Protocol = function()
{
	var protocol = {};
	var debug = 0;
	
	var lysatesIsValid = false;

	// Page initialization
	protocol.initPage = function()
	{
		var pageId = Doc.getPageId();
		if (pageId == 'protocol')
		{
			Buttons.addClickHandler('print-button', Wizard.goPrint);	
			Doc.show('all-protocol');
			
			var url = '../Extraction.servlet?ID='+App.getSessionId();
			url += '&cmd=GetLatestReagentLotNumbers';		
			Wizard.showLoadingAnimation('Loading recent reagent lot numbers...');
			Wizard.asyncJsonRequest(url, protocol.reagentsLoaded);
			
			Events.addEventHandler('showReagents', 'click', protocol.toggleReagents);
		}
		else
		{
			Events.addEventHandler('lysates', 'change', protocol.lysatesOnChange);
			Events.addEventHandler('reextraction', 'change', protocol.lysatesOnChange);
			Buttons.addClickHandler('downloadLabels', protocol.downloadLabels);
			Buttons.addClickHandler('gocreate', protocol.viewProtocol);

			var url = '../Extraction.servlet?ID='+App.getSessionId();
			url += '&cmd=GetUnprocessedLysates&includeReprocess=1';
			Wizard.showLoadingAnimation('Loading Lysate extracts...');
			Wizard.asyncJsonRequest(url, protocol.initializeStep1);
		}
	}


	protocol.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];	
		var lysates = response.lysates;
		
		Doc.element('reserve').value = response.ReservedBy.id;
		Doc.element('YellowLabel').value = response.YellowLabel.id;
		Doc.element('AutoProcessing').value = response.AutoProcessing.id;
		
		var lysateList = frm.lysates;
		var reextractList = frm.reextraction;
		if (lysates.length > 0)
		{
			// Check if we have any lysate reserved by the current user
			var currentUser = Data.get(lysateList, 'current-user');
			var hasReserved = false;
			var numYellow = 0;
			for (var i=0; i < lysates.length; i++)
			{
				var lysate = lysates[i];
				if (lysate.ReservedBy == currentUser) hasReserved = true;
				if (lysate.specimen.YellowLabel != null) numYellow++;
			}
			
			var numSelected = 0;
			var maxNonYellow = 12-numYellow;
			for (var i=0; i < lysates.length; i++)
			{
				var lysate = lysates[i];
				var isYellow = lysate.specimen.YellowLabel != null;
				lysate.isReProcess = lysate.AutoProcessing == 'ReProcess';
				var name = (i+1) + ': ' + Strings.encodeTags(lysate.name);
				if (lysate.bioWell)
				{
					name += ' -- ' + Strings.encodeTags(lysate.bioWell.bioPlate.name + ' (' + lysate.bioWell.location+')');
				}
				if (lysate.ReservedBy)
				{
					name += ' ['+lysate.ReservedBy+']';
				}
				
				// We only auto-select items reserved by the current user (if it has any)
				// and never any items reserved by other users
				var selected = false;
				if (numSelected < 12)
				{
					if (hasReserved)
					{
						selected = lysate.ReservedBy == currentUser;
					}
					else
					{
						selected = isYellow || maxNonYellow > 0;
					}
					if (selected) 
					{
						numSelected++;
						if (!isYellow) maxNonYellow--;
					}
				}
				
				var option = new Option(name, lysate.id, selected, selected);
				if (isYellow) option.className = 'yellow';
				option.lysate = lysate;
				if (lysate.isReProcess)
				{
					reextractList.options[reextractList.length] = option;
				}
				else
				{
					lysateList.options[lysateList.length] = option;
				}
			}
			if (lysateList.length == 0) Doc.hide('regularFlowRow');
			if (reextractList.length > 0) Doc.show('reExtractionRow');
			protocol.lysatesOnChange();
		}
		else
		{	
			Wizard.setFatalError('No Lysate available for processing.');
			return;
		}
		
		Doc.show('step-1');
		Doc.show('gocreate');
	}
	

	// Add pools to the pools list based on the bioplate selection
	protocol.lysatesOnChange = function()
	{
		var frm = document.forms['reggie'];
		lysatesIsValid = false;
		Wizard.setInputStatus('lysates');
		Wizard.setInputStatus('reextraction');
		
		var numLysates = 0;
		var numReextraction = 0;
		var numUnselectedYellowLysates = 0;
		var numUnselectedYellowReextraction = 0;
		
		for (var i = 0; i < frm.lysates.length; i++)
		{
			var lysate = frm.lysates[i].lysate;
			if (frm.lysates[i].selected)
			{
				numLysates++;
			}
			else if (lysate.specimen.YellowLabel != null)
			{
				numUnselectedYellowLysates++;
			}
		}
		for (var i = 0; i < frm.reextraction.length; i++)
		{
			var lysate = frm.reextraction[i].lysate;
			if (frm.reextraction[i].selected)
			{
				numReextraction++;
			}
			else if (lysate.specimen.YellowLabel != null)
			{
				numUnselectedYellowReextraction++;
			}
		}
		
		if (numReextraction == 0 && numLysates == 0)
		{
			Wizard.setInputStatus('lysates', 'invalid', 'At least one lysate must be selected');
			Wizard.setInputStatus('reextraction', 'invalid', 'At least one lysate must be selected');
			return;
		}

		if (numReextraction+numLysates > 12)
		{
			if (numLysates > 0) Wizard.setInputStatus('lysates', 'invalid', 'Must not select more than 12 items.');
			if (numReextraction > 0) Wizard.setInputStatus('reextraction', 'invalid', 'Must not select more than 12 items.');
			return;
		}
		
		lysatesIsValid = true;
		if (numUnselectedYellowLysates > 0)
		{
			Wizard.setInputStatus('lysates', 'warning', 'Not all YellowLabel specimen are selected.');
		}
		else
		{
			Wizard.setInputStatus('lysates', 'valid');
		}
		if (numUnselectedYellowReextraction > 0)
		{
			Wizard.setInputStatus('reextraction', 'warning', 'Not all YellowLabel specimen are selected.');
		}
		else
		{
			Wizard.setInputStatus('reextraction', 'valid');
		}
		
		if (numReextraction > 0)
		{
			Doc.removeClass('downloadLabels', 'disabled');
		}
		else
		{
			Doc.addClass('downloadLabels', 'disabled');
		}

	}
	
	protocol.downloadLabels = function()
	{
		var frm = document.forms['reggie'];
		
		var lysates = [];
		for (var i = 0; i < frm.reextraction.length; i++)
		{
			var lysate = frm.reextraction[i].lysate;
			if (frm.reextraction[i].selected)
			{
				lysates[lysates.length] = lysate.id;
			}
		}
		
		var frm = document.forms['reggie'];
		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=DownloadLabelFile';
		url += '&lysates=' + encodeURIComponent(lysates.join(','));
		window.open(url);
		
	}
	
	protocol.viewProtocol = function()
	{
		var frm = document.forms['reggie'];
		if (!lysatesIsValid)
		{
			Wizard.notifyError();
			return;
		}
		frm.submit();
	}

	protocol.reagentsLoaded = function(response)
	{
		var rna = response.rna;
		var lysate = response.lysate;
		if (rna) 
		{
			protocol.fillLatestReagents(rna);
			Doc.element('latestQiaCubeDate').innerHTML = Reggie.reformatDate(rna.QiaCubeDate);
			Doc.show('showLatestReagents', 'inline');
		}
		if (lysate) protocol.fillLatestReagents(lysate);
	}
	
	protocol.fillLatestReagents = function(item)
	{
		for (var rg in item)
		{
			if (item[rg] != null)
			{
				var e = Doc.element(rg);
				if (e)
				{
					e.innerHTML = Strings.encodeTags(item[rg]);
				}
				
			}
		}
	}
	
	protocol.toggleReagents = function(event)
	{
		Doc.addOrRemoveClass(document.body, 'hide-reagents', !event.currentTarget.checked);
	}
	
	return protocol;
}();

Doc.onLoad(Protocol.initPage);

